define(["dojo/_base/declare", "dojo/_base/lang", "dojo/Deferred", "aps/ResourceStore" ],
    function(declare, lang, Deferred, Store){

        var utils = {};
        utils["compareObj"] = function( x, y ) {
            if ( x === y ) return true;
            // if both x and y are null or undefined and exactly the same

            if ( ! ( x instanceof Object ) || ! ( y instanceof Object ) ) return false;
            // if they are not strictly equal, they both need to be Objects

            if ( x.constructor !== y.constructor ) return false;
            // they must have the exact same prototype chain, the closest we can do is
            // test there constructor.

            for ( var p in x ) {
                if (x.hasOwnProperty( p )){
                      // other properties were tested using x.constructor === y.constructor

                    if ( ! y.hasOwnProperty( p ) ) return false;
                      // allows to compare x[ p ] and y[ p ] when set to undefined

                    if ( x[ p ] === y[ p ] ) continue;
                      // if they have the same strict value or identity then they are equal

                    if ( typeof( x[ p ] ) !== "object" ) return false;
                      // Numbers, Strings, Functions, Booleans must be strictly equal

                    if ( ! utils.compareObj( x[ p ],  y[ p ] ) ) return false;
                      // Objects and Arrays must be tested recursively
                }
            }

            for ( p in y ) {
                if ( y.hasOwnProperty( p ) && ! x.hasOwnProperty( p ) ) return false;
                  // allows x[ p ] to be set to undefined
            }
            return true;
        };

        return declare([Store], {
            prepareCallback: function (items, workCallback) { workCallback(items); },
            itemCallback: null,     // function (item)
            itemsCallback: null,    // function (items)
            finalItemsCallback: function() {}, //function(items)
            errorCallback: function() {}, //function(items)

            queryCallback: function () {
                /*
                var trimQuery = function (query) {
                    for (var key in query) {
                        if (query.hasOwnProperty(key) && typeof query[key] === "string" && query[key].indexOf("*") == -1) {
                            query[key] = query[key].replace(/^\s+/g, '').replace(/\s+$/g, '');
                        }
                    }
                }; */
                //trimQuery(remoteQuery);
                //trimQuery(resultQuery);
            }, //function (remoteQuery, remoteOptions, resultQuery, resultOptions)

            _expirationTime: null,
            _cached: null,
            _cachedQuery: null,
            _cachedOptions: null,
            warmUp: function(expirationTimeInSeconds, query, options) {
                var expirationTime = new Date();
                expirationTime.setTime(expirationTime.getTime() + expirationTimeInSeconds * 1000);
                this._expirationTime = null;
                this._cachedQuery = lang.clone(query);
                this._cachedOptions = lang.clone(options);
                this._cached = this.query(query, options);
                this._expirationTime = expirationTime;
                return this._cached;
            },

            query: function(query, options) {
                var self = this;

                options = options || {};

                if (this._expirationTime != null && utils.compareObj(this._cachedQuery, query) && utils.compareObj(this._cachedOptions, options)) {
                    if (this._expirationTime > new Date()) {
                        //console.log("CACHED!!!");
                        return this._cached;
                    } else {
                        //console.log("CACHE EXPIRED!!!");
                        //console.log(this._expirationTime);
                        //console.log(new Date());
                        this._expirationTime = null;
                        this._cachedQuery = null;
                        this._cachedOptions = null;
                        this._cached = null;
                    }
                }
                // console.log("NOT CACHED!!!");
                // console.log(this._expirationTime);
                // console.log(this._cachedQuery);
                // console.log(query);
                // console.log(this._cachedOptions);
                // console.log(options);

                var originalQueryArg = lang.clone(query);
                var originalOptionsArg = lang.clone(options);
                if (self.queryCallback != null) {
                    self.queryCallback(query, options, originalQueryArg, originalOptionsArg);
                }

                /*jshint noarg:true */
                var originalQueryResult = this.inherited(arguments);

                var result = new Deferred();
                result.total = new Deferred();

                originalQueryResult.then(
                    function (items) {
                        self.prepareCallback(items, function (list) {
                            if (self.itemCallback != null) {
                                for (var i=0; i<list.length; i++) {
                                    self.itemCallback(list[i]);
                                }
                            }
                            var newList = self.itemsCallback == null ? null : self.itemsCallback(list, originalQueryArg, originalOptionsArg);
                            if (newList == null) {
                                newList = list;
                            }
                            if (self.finalItemsCallback != null) {
                                self.finalItemsCallback(newList);
                            }
                            originalQueryResult.total.then(function (total) {
                                if ("total" in newList) {
                                    total = newList.total;
                                }
                                else if (total === 0 && newList.length > 0) { // w/a for result without Content-Range
                                    total = newList.length;
                                }
                                result.total.resolve(total);
                            });
                            result.resolve(newList);
                        });
                    },
                    function (err) {
                        self.errorCallback(err);
                        result.reject(err);
                    });

                return result;
            }
        });
});