define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/Deferred",
        "aps/Store",
        "./CustomizableStore.js",
        "./utils.js"
], function (
    declare,
    lang,
    Deferred,
    Store,
    CustomizableStore,
    utils
) {

return declare([CustomizableStore], {
	customColumns: null,
	constructor: function() {
		if (this.customColumns != null) {
			for (var key in this.customColumns) {
				if(this.customColumns.hasOwnProperty(key)){
					(function (columnKey, column) {
						if ("map" in column) {
							if (!("get" in column)) {
								var map = column.map;
								var targetAttr = "targetAttr" in column ? column.targetAttr : null;
								if (map instanceof Store) {
									column["get"] = targetAttr == null ?
										function (item) {
											return  map.get(item[column.sourceAttr]);
										} :
										function (item) {
											return  map.get(item[column.sourceAttr])[targetAttr];
										};
								} else {
									column["get"] = targetAttr == null ?
										function (item) {
											return column.sourceAttr in item ? map[item[column.sourceAttr]] : undefined;
										} :
										function (item) {
											return (column.sourceAttr in item) && (item[column.sourceAttr] in map) ? map[item[column.sourceAttr]][targetAttr] : undefined;
										};
								}
							}
							if (!("sort" in column) && "additional_sort" in column) {
								column["sort"] = function (a, b) {
									if (a[columnKey] == b[columnKey]) {
										if (a[column.sourceAttr] == b[column.sourceAttr])
											return column["additional_sort"](a, b);
										return utils.compareStr(a[column.sourceAttr], b[column.sourceAttr]);
									}
									return utils.compareStr(a[columnKey], b[columnKey]);
								};
							}
						}
					})(key, this.customColumns[key]);
				}
			}
		}
	},
	queryCallback: function(query, options) {
		if (this.customColumns != null &&
			options != null &&
			"sort" in options &&
			options.sort.length == 1 &&
			options.sort[0].attribute in this.customColumns) {
			delete options["sort"];
			delete options["start"];
			delete options["count"];
		}
	},
	itemCallback: function(item) {
		if (this.customColumns != null) {
			for (var key in this.customColumns) {
				if ("get" in this.customColumns[key]) {
					item[key] = this.customColumns[key].get(item);
				}
			}
		}
	},
	itemsCallback: function(items, query, options) {
		if (this.customColumns != null &&
			options != null &&
			"sort" in options &&
			options.sort.length == 1 &&
			options.sort[0].attribute in this.customColumns) {
			var ths = this;
			items.sort(function (a, b) {
				return ths.customColumns[options.sort[0].attribute].sort(a, b) * (options.sort[0].descending ? -1 : 1);
			});
			var count = "count" in options ? options["count"] : items.length;
			var start = "start" in options ? options["start"] : 0;
			if (count != items.length || start !== 0) {
				var result = [];
				for (var i = start; i < start + count && i < items.length; i++) {
					result.push(items[i]);
				}
				delete options["start"];
				delete options["count"];
				return result;
			}
		}
		return items;
	}
});

});